// C/C++ headers

#include <stdio.h>
#include <sstream>
#include <string>

using std::ostringstream;
using std::string;

// Our headers

#include "ComponentSupplier.h"
#include "MessageReceiver.h"
#include "Widget.h"
#include "StringMap.h"
#include "QComponent.h"
#include "QMainComponent.h"

// Qt headers/forward references

class QWidget;

/*****************************************************************************
 * Definition of class: ComponentSupplier
 *****************************************************************************/

ComponentSupplier::ComponentSupplier(
   MessageReceiver &receiver,
   Widget *w,
   StringMap *map
)  :
   mIndex(0),
   mMap(map),
   mReceiver(receiver)
{
   w->acquireComponent(w->getName(), this);
}

ComponentSupplier::~ComponentSupplier()
{
   // Nothing to do
}

Component *ComponentSupplier::getComponent(const char *name)
{
   QWidget *q     = (QWidget *)mMap->get(name);
   Component *ptr = NULL;

   if(q)
   {
      if(mIndex)
         ptr = new QComponent(mIndex, q);
      else
         ptr = new QMainComponent(mIndex, q);

      mIndex++;
   }
   else
   {
      ostringstream o;

      o << "No component found for widget: [" << name << "]";

      string s = o.str();
      mReceiver.accept(s.c_str());
   }

   return(ptr);
}
