// C/C++ headers

#include <stdio.h>
#include <string>

using std::string;

// QALE headers

#include "DefineVisitor.h"
#include "DefineNode.h"
#include "Widget.h"
#include "MessageReceiver.h"

/*****************************************************************************
 * Definition of class: DefineVisitor
 *****************************************************************************/

DefineVisitor::DefineVisitor(MessageReceiver &receiver)
   : mReceiver(receiver)
{
   // Nothing to do
}

DefineVisitor::~DefineVisitor()
{
   // Nothing to do
}

void DefineVisitor::visited(AttributeNode *) {}
void DefineVisitor::visited(BinaryNode *)    {}
void DefineVisitor::visited(ConstantNode *)  {}

void DefineVisitor::visited(DefineNode *defineNode)
{
   char *qualifiedName = defineNode->getQualifiedName();
   char *defineName    = defineNode->getDefineName();
   Node *resolved      = NULL;

   if(*qualifiedName)
   {
      Widget *theWidget = findWidget(qualifiedName);

      if(theWidget)
         resolved = (Node *)theWidget->getDefine(defineName);
   }
   else
   {
      Widget *theWidget = getCurrentWidget();

      while((theWidget = theWidget->getParent()) != 0)
      {
         if((resolved = (Node *)theWidget->getDefine(defineName)) != 0)
            break;
      }
   }

   if(!resolved)
   {
      string errMsg = getErrorMessage("undefined variable", defineNode);
      mReceiver.accept(errMsg.c_str());
   }
   else
   {
      defineNode->setNode(resolved->copy());
   }
}

void DefineVisitor::visited(PropertyNode *) {}
void DefineVisitor::visited(MinMaxNode *) {}
void DefineVisitor::visited(UnaryNode *)  {}
