// C/C++ headers

#include <string.h>
#include <stdio.h>

#include <sstream>

using std::ostringstream;

// Our Headers

#include "Visitor.h"
#include "Widget.h"
#include "Node.h"
#include "cpp_assert.h"

/*****************************************************************************
 * Definition of class: Visitor
 *****************************************************************************/

Visitor::Visitor() :
   mCurrentWidget(NULL),
   mField(UNKNOWN)
{
   // Nothing to do
}

Visitor::~Visitor()
{
   // Nothing to do
}

void Visitor::setCurrentWidget(Widget *w)
{
   mCurrentWidget = w;
}

Widget *Visitor::getCurrentWidget()
{
   return(mCurrentWidget);
}

Widget *Visitor::findWidget(const char *qualifiedName)
{
   // We begin with our perspective being from that of the container
   // that holds our current widget

   Widget *current = getCurrentWidget()->getParent();
   char *copy      = new char[strlen(qualifiedName) + 1];

   strcpy(copy, qualifiedName);

   char *token = strtok(copy, ".");
   int i       = 0;

   while(token)
   {
      // If our currentWidget was our top-level widget then the only
      // acceptable qualification would be the name of our top-level
      // widget, deal with this case now

      if(i == 0 && current == NULL)
      {
         Widget *topLevelWidget = getCurrentWidget();

         if(strcmp(topLevelWidget->getName(), token) == 0)
            current = topLevelWidget;

         else
            break;
      }
      else
      {
         // token will be one of:
         //   (a) [parent]
         //   (b) [this]
         //   (c) widgetName

         if(*token == '[')
         {
            if(token[1] == 'p' || token[1] == 'p')         
            {
               if((current = current->getParent()) == NULL)
               {
                  // We have no parent, we're done

                  break;
               }
            }
            else
            {
               // token was "[this]", nothing to do
            }
         }
         else
         {
            if((current = current->getChild(token)) == NULL)
            {
               // No child found with the specified name, we're done

               break;
            }
         }
      }

      i++;
      token = strtok(NULL, ".");
   }

   delete [] copy;

   return(current);
}

void Visitor::setField(Field field)
{
   mField = field;
}

const char *Visitor::fieldAsString() const
{
   char *asString = NULL;

   switch(mField)
   {
      case SIZE_WIDTH : asString = "width";      break;
      case SIZE_HEIGHT: asString = "height";     break;
      case X_LOCATION : asString = "x location"; break;
      case Y_LOCATION : asString = "y location"; break;

      default:
      {
         // Shouldn't be here

         CPP_ASSERT(0);
      }
      break;
   }

   return(asString);
}

string Visitor::getErrorMessage(const char *text, const Node *n)
{
   ostringstream errStream;

   errStream << "The " << fieldAsString() << " of widget: "
             << getCurrentWidget()->getQualifiedName() << "\n"
             << "depends upon " << text << ": " << *n;

   return(errStream.str());
}

Visitor::Field Visitor::getField()
{
   return(mField);
}
